@@ -1065,8 +1065,6 @@ extern void (*_pSLang_Matrix_Multiply)(void);
 extern int _pSLarray_next_index (SLindex_Type *, SLindex_Type *, unsigned int);
 
 extern int _pSLarray_init_slarray (void);
-extern SLang_Array_Type *
-SLang_create_array1 (SLtype, int, VOID_STAR, SLindex_Type *, unsigned int, int);
 
 extern int _pSLassoc_aput (SLtype, unsigned int);
 extern int _pSLassoc_aget (SLtype, unsigned int);
@@ -1570,6 +1568,14 @@ extern int _pSLang_check_signals_hook (VOID_STAR);
 # define _INLINE_
 #endif
 
+#if SLANG_OPTIMIZE_FOR_SPEED && defined(__GNUC__) && (__GNUC__ >= 3)
+# define IF_LIKELY(x) if (__builtin_expect((x),1))
+# define IF_UNLIKELY(x) if (__builtin_expect ((x), 0))
+#else
+# define IF_LIKELY(x) if (x)
+# define IF_UNLIKELY(x) if (x)
+#endif
+
 /* This is a macro that permits:
  *
  *   extern fun (void **addr);
