@@ -22,8 +22,8 @@ Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307,
 USA.  
 */
 
-#define SLANG_VERSION 20200
-#define SLANG_VERSION_STRING "2.2.0"
+#define SLANG_VERSION 20202
+#define SLANG_VERSION_STRING "2.2.2"
 /* #ifdef __DATE__ */
 /* # define SLANG_VERSION_STRING SLANG_VERSION_STRING0 " " __DATE__ */
 /* #else */
@@ -745,7 +745,12 @@ SL_EXTERN int SLclass_set_foreach_functions (
   int (*)(SLtype, SLang_Foreach_Context_Type *),   /* foreach method */
   void (*)(SLtype, SLang_Foreach_Context_Type *));/* close method */
 
-   
+/* When an array is created, this callback function is called to initialize the data
+ * at the specified address.
+ */
+SL_EXTERN int SLclass_set_aelem_init_function (SLang_Class_Type *cl, int (*f)(SLtype, VOID_STAR));
+
+
 /* Typecast object on the stack to type p1.  p2 and p3 should be set to 1 */
 SL_EXTERN int SLclass_typecast (SLtype, int, int);
 
@@ -893,6 +898,7 @@ SL_EXTERN int SLang_pop_array (SLang_Array_Type **atp, int convert_scalar);
 SL_EXTERN int SLang_push_array (SLang_Array_Type *at, int do_free);
 SL_EXTERN void SLang_free_array (SLang_Array_Type *at);
 SL_EXTERN SLang_Array_Type *SLang_create_array (SLtype, int, VOID_STAR, SLindex_Type *, unsigned int);
+SL_EXTERN SLang_Array_Type *SLang_create_array1 (SLtype, int, VOID_STAR, SLindex_Type *, unsigned int, int);
 SL_EXTERN SLang_Array_Type *SLang_duplicate_array (SLang_Array_Type *);
 SL_EXTERN int SLang_get_array_element (SLang_Array_Type *, SLindex_Type *, VOID_STAR);
 SL_EXTERN int SLang_set_array_element (SLang_Array_Type *, SLindex_Type *, VOID_STAR);
@@ -2151,7 +2157,7 @@ SL_EXTERN int SLang_get_int_size (SLtype);
 
 #define SLANG_UNARY_OP_MAX	0x2B
 
-SL_EXTERN SLFUTURE_CONST char *SLang_Error_Message;
+/* SL_EXTERN SLFUTURE_CONST char *SLang_Error_Message; */
 
 SL_EXTERN int SLadd_intrinsic_variable (SLFUTURE_CONST char *, VOID_STAR, SLtype, int);
 SL_EXTERN int SLadd_intrinsic_function (SLFUTURE_CONST char *, FVOID_STAR, SLtype, unsigned int,...);
