@@ -1,5 +1,5 @@
 /*
-Copyright (C) 2004, 2005, 2006, 2007, 2008 John E. Davis
+Copyright (C) 2004-2009 John E. Davis
 
 This file is part of the S-Lang Library.
 
@@ -44,6 +44,8 @@ USA.
 # endif
 #endif
 
+#include <ctype.h>
+
 #include "slang.h"
 #include "_slang.h"
 
@@ -299,44 +301,51 @@ SLarith_get_to_double_fun (SLtype type, unsigned int *sizeof_type)
 #endif				       /* LONG_IS_NOT_INT */
 
 #ifdef HAVE_LONG_LONG
-#define GENERIC_BINARY_FUNCTION llong_llong_bin_op
-#define GENERIC_BIT_OPERATIONS
-#define GENERIC_TYPE long long
-#define POW_FUNCTION(a,b) pow((double)(a),(double)(b))
-#define POW_RESULT_TYPE double
-#define MOD_FUNCTION(a,b) ((a) % (b))
-#define TRAP_DIV_ZERO	1
-#define GENERIC_UNARY_FUNCTION llong_unary_op
-#define GENERIC_ARITH_UNARY_FUNCTION llong_arith_unary_op
-#define ABS_FUNCTION(a) (((a) >= 0) ? (a) : -(a))
-#define SIGN_FUNCTION(x) (((x) > 0) ? 1 : (((x) < 0) ? -1 : 0))
-#if SLANG_OPTIMIZE_FOR_SPEED
-# define SCALAR_BINARY_FUNCTION llong_llong_scalar_bin_op
-#endif
-#define PUSH_SCALAR_OBJ_FUN(x) SLclass_push_llong_obj(SLANG_LLONG_TYPE,(x))
-#define PUSH_POW_OBJ_FUN(x) SLclass_push_double_obj(SLANG_DOUBLE_TYPE, (x))
-#define CMP_FUNCTION llong_cmp_function
-#include "slarith.inc"
+# if LLONG_IS_NOT_LONG
+#  define GENERIC_BINARY_FUNCTION llong_llong_bin_op
+#  define GENERIC_BIT_OPERATIONS
+#  define GENERIC_TYPE long long
+#  define POW_FUNCTION(a,b) pow((double)(a),(double)(b))
+#  define POW_RESULT_TYPE double
+#  define MOD_FUNCTION(a,b) ((a) % (b))
+#  define TRAP_DIV_ZERO	1
+#  define GENERIC_UNARY_FUNCTION llong_unary_op
+#  define GENERIC_ARITH_UNARY_FUNCTION llong_arith_unary_op
+#  define ABS_FUNCTION(a) (((a) >= 0) ? (a) : -(a))
+#  define SIGN_FUNCTION(x) (((x) > 0) ? 1 : (((x) < 0) ? -1 : 0))
+#  if SLANG_OPTIMIZE_FOR_SPEED
+#   define SCALAR_BINARY_FUNCTION llong_llong_scalar_bin_op
+#  endif
+#  define PUSH_SCALAR_OBJ_FUN(x) SLclass_push_llong_obj(SLANG_LLONG_TYPE,(x))
+#  define PUSH_POW_OBJ_FUN(x) SLclass_push_double_obj(SLANG_DOUBLE_TYPE, (x))
+#  define CMP_FUNCTION llong_cmp_function
+#  include "slarith.inc"
 
-#define GENERIC_BINARY_FUNCTION ullong_ullong_bin_op
-#define GENERIC_BIT_OPERATIONS
-#define GENERIC_TYPE unsigned long long
-#define GENERIC_TYPE_IS_UNSIGNED
-#define POW_FUNCTION(a,b) pow((double)(a),(double)(b))
-#define POW_RESULT_TYPE double
-#define MOD_FUNCTION(a,b) ((a) % (b))
-#define TRAP_DIV_ZERO	1
-#define GENERIC_UNARY_FUNCTION ullong_unary_op
-#define GENERIC_ARITH_UNARY_FUNCTION ullong_arith_unary_op
-#define ABS_FUNCTION(a) (a)
-#define SIGN_FUNCTION(x) (((x) > 0) ? 1 : 0)
-#if SLANG_OPTIMIZE_FOR_SPEED
-# define SCALAR_BINARY_FUNCTION ullong_ullong_scalar_bin_op
-#endif
-#define PUSH_SCALAR_OBJ_FUN(x) SLclass_push_llong_obj(SLANG_ULLONG_TYPE,(long long)(x))
-#define PUSH_POW_OBJ_FUN(x) SLclass_push_double_obj(SLANG_DOUBLE_TYPE, (x))
-#define CMP_FUNCTION ullong_cmp_function
-#include "slarith.inc"
+#  define GENERIC_BINARY_FUNCTION ullong_ullong_bin_op
+#  define GENERIC_BIT_OPERATIONS
+#  define GENERIC_TYPE unsigned long long
+#  define GENERIC_TYPE_IS_UNSIGNED
+#  define POW_FUNCTION(a,b) pow((double)(a),(double)(b))
+#  define POW_RESULT_TYPE double
+#  define MOD_FUNCTION(a,b) ((a) % (b))
+#  define TRAP_DIV_ZERO	1
+#  define GENERIC_UNARY_FUNCTION ullong_unary_op
+#  define GENERIC_ARITH_UNARY_FUNCTION ullong_arith_unary_op
+#  define ABS_FUNCTION(a) (a)
+#  define SIGN_FUNCTION(x) (((x) > 0) ? 1 : 0)
+#  if SLANG_OPTIMIZE_FOR_SPEED
+#   define SCALAR_BINARY_FUNCTION ullong_ullong_scalar_bin_op
+#  endif
+#  define PUSH_SCALAR_OBJ_FUN(x) SLclass_push_llong_obj(SLANG_ULLONG_TYPE,(long long)(x))
+#  define PUSH_POW_OBJ_FUN(x) SLclass_push_double_obj(SLANG_DOUBLE_TYPE, (x))
+#  define CMP_FUNCTION ullong_cmp_function
+#  include "slarith.inc"
+# else
+#  define llong_llong_bin_op long_long_bin_op
+#  define ullong_ullong_bin_op ulong_ulong_bin_op
+#  define llong_llong_scalar_bin_op long_long_scalar_bin_op
+#  define ullong_ullong_scalar_bin_op ulong_ulong_scalar_bin_op
+# endif				       /* LLONG_IS_NOT_LONG */
 #endif				       /* HAVE_LONG_LONG */
 
 #if SLANG_HAS_FLOAT
@@ -710,13 +719,13 @@ int SLang_pop_ulong (unsigned long *i)
 }
 
 #ifdef HAVE_LONG_LONG
-
+#if LLONG_IS_NOT_LONG
 static void llong_byte_code_destroy (SLtype unused, VOID_STAR ptr)
 {
    (void) unused;
    SLfree (*(char **) ptr);
 }
-
+#endif
 int SLang_pop_long_long (long long *i)
 {
    return integer_pop (_pSLANG_LLONG_TYPE, (VOID_STAR) i);
@@ -907,18 +916,195 @@ static int float_pop (SLtype unused, VOID_STAR ptr)
 
 #if SLANG_HAS_FLOAT
 static char Double_Format[16] = "%g";
+static char *Double_Format_Ptr = NULL;
+static unsigned int Double_Format_Expon_Threshold = 6;
 
-void _pSLset_double_format (SLCONST char *s)
+void _pSLset_double_format (SLCONST char *fmt)
 {
-   if (strlen (s) >= sizeof (Double_Format))
-     return;
+   /* The only forms accepted by this function are:
+    * "%[+ ][width][.precision][efgS]"
+    */
+   SLCONST char *s = fmt;
+   int precision = 6;
 
-   strcpy (Double_Format, s);
+   if (*s++ != '%')
+     return;
+   
+   /* 0 or more flags */
+   while ((*s == '#') || (*s == '0') || (*s == '-') 
+	  || (*s == ' ') || (*s == '+'))
+     s++;
+
+   /* field width */
+   while (isdigit (*s)) s++;
+   
+   /* precision */
+   if (*s == '.')
+     {
+	s++;
+	precision = 0;
+	while (isdigit (*s))
+	  {
+	     precision = precision * 10 + (*s - '0');
+	     s++;
+	  }
+	if (precision < 0)
+	  precision = 6;
+     }
+
+   if ((*s == 'e') || (*s == 'E')
+       || (*s == 'f') || (*s == 'F')
+       || (*s == 'g') || (*s == 'G'))
+     {
+	s++;
+	if (*s != 0)
+	  return;		       /* more junk-- unacceptable */
+
+	if (strlen (fmt) >= sizeof (Double_Format))
+	  return;
+
+	strcpy (Double_Format, fmt);
+	Double_Format_Ptr = Double_Format;
+	return;
+     }
+   
+   if ((*s == 'S') || (*s == 's'))
+     {
+	s++;
+	if (*s != 0)
+	  return;
+
+	Double_Format_Ptr = NULL;
+	Double_Format_Expon_Threshold = precision;
+	return;
+     }
+
+   /* error */
 }
 
 SLCONST char *_pSLget_double_format (void)
 {
-   return Double_Format;
+   if (Double_Format_Ptr == NULL)
+     return "%S";
+   
+   return Double_Format_Ptr;
+}
+
+static void check_decimal (char *buf, unsigned int buflen, double x)
+{
+   char *b, *bstart = buf, *bufmax = buf + buflen;
+   char ch;
+   unsigned int count = 0, expon;
+   int has_point = 0;
+   unsigned int expon_threshold = Double_Format_Expon_Threshold;
+
+   if (*bstart == '-')
+     bstart++;
+
+   b = bstart;
+   while (1)
+     {
+	ch = *b;
+	if (isdigit (ch))
+	  {
+	     count++;
+	     b++;
+	     continue;
+	  }
+	if (ch == 0)
+	  break;		       /* all digits */
+
+	if (ch != '.')
+	  return;			       /* something else */
+
+	/* We are at a decimal point.  If expon > 1, then buf does not contain 
+	 * an exponential formatted quantity.
+	 */
+	if (count <= expon_threshold)
+	  return;
+	/* We have something like: 1234567.123, which we want to
+	 * write as 1.234567123e+6
+	 */
+	b += strlen(b);
+	has_point = 1;
+	break;			       /* handle below */
+     }
+
+   /* We get here only when *b==0. */
+   
+   if ((has_point == 0) && (count <= 6))
+     {
+	if (b + 3 >= bufmax)
+	  {
+	     sprintf (buf, "%e", x);
+	     return;
+	  }
+	*b++ = '.';
+	*b++ = '0';
+	*b = 0;
+	return;
+     }
+
+   expon = count-1;
+   
+   /* Now add on the exponent.  First move the decimal point but drop trailing 0s */
+   while ((count > 1) && (*(b-1) == '0'))
+     {
+	b--;
+	count--;
+     }
+
+   if (count > 1)
+     {	
+	while (count > 1)
+	  {
+	     bstart[count] = bstart[count-1];
+	     count--;
+	  }
+	bstart[count] = '.';
+	if (has_point == 0)
+	  b++;
+     }
+
+   if (EOF == SLsnprintf (b, bufmax-b, "e+%02d", expon))
+     sprintf (buf, "%e", x);
+}
+	
+static void default_format_double (double x, char *buf, unsigned int buflen)
+{
+   if (EOF == SLsnprintf (buf, buflen, "%.16g", x))
+     {
+	sprintf (buf, "%e", x);
+	return;
+     }
+
+   if (atof (buf) != x)
+     {
+	if (EOF == SLsnprintf (buf, buflen, "%.17g", x))
+	  {
+	     sprintf (buf, "%e", x);
+	     return;
+	  }
+     }
+   check_decimal (buf, buflen, x);
+}
+
+static void default_format_float (float x, char *buf, unsigned int buflen)
+{
+   if (EOF == SLsnprintf (buf, buflen, "%.8g", x))
+     {
+	sprintf (buf, "%e", x);
+	return;
+     }
+   if ((float) atof (buf) != x)
+     {
+	if (EOF == SLsnprintf (buf, buflen, "%.9g", x))
+	  {
+	     sprintf (buf, "%e", x);
+	     return;
+	  }
+     }
+   check_decimal (buf, buflen, x);
 }
 #endif
 
@@ -969,11 +1155,15 @@ static char *arith_string (SLtype type, VOID_STAR v)
 #endif
 #if SLANG_HAS_FLOAT
       case SLANG_FLOAT_TYPE:
-	if (EOF == SLsnprintf (buf, sizeof (buf), Double_Format, *(float *) v))
+	if (Double_Format_Ptr == NULL)
+	  default_format_float (*(float *)v, buf, sizeof(buf));
+	else if (EOF == SLsnprintf (buf, sizeof (buf), Double_Format, *(float *) v))
 	  sprintf (s, "%e", *(float *) v);
 	break;
       case SLANG_DOUBLE_TYPE:
-	if (EOF == SLsnprintf (buf, sizeof (buf), Double_Format, *(double *) v))
+	if (Double_Format_Ptr == NULL)
+	  default_format_double (*(double *)v, buf, sizeof(buf));
+	else if (EOF == SLsnprintf (buf, sizeof (buf), Double_Format, *(double *) v))
 	  sprintf (s, "%e", *(double *) v);
 	break;
 #endif
@@ -1016,11 +1206,13 @@ static int push_long_literal (SLtype type, VOID_STAR ptr)
 #endif
 
 #ifdef HAVE_LONG_LONG
+#if LLONG_IS_NOT_LONG
 static int push_llong_literal (SLtype type, VOID_STAR ptr)
 {
    return SLclass_push_llong_obj (type, **(long long **)ptr);
 }
 #endif
+#endif
 typedef struct
 {
    SLFUTURE_CONST char *name;
