@@ -1,6 +1,6 @@
 /* cmd line facility for slang */
 /*
-Copyright (C) 2004, 2005, 2006, 2007, 2008 John E. Davis
+Copyright (C) 2004-2009 John E. Davis
 
 This file is part of the S-Lang Library.
 
@@ -238,7 +238,7 @@ int SLcmd_execute_string (SLFUTURE_CONST char *str, SLcmd_Cmd_Table_Type *table)
 		  ch = *b;
 		  if (ch == '\\')
 		    {
-		       if (NULL == _pSLexpand_escaped_char (b, &ch, NULL))
+		       if (NULL == _pSLexpand_escaped_char (b, b+len, &ch, NULL))
 			 goto error;
 		    }
 		  sprintf (buf, "%lu", (unsigned long)ch);
