@@ -2,7 +2,7 @@
  * links to the library.
  */
 /*
-Copyright (C) 2004, 2005, 2006, 2007, 2008 John E. Davis
+Copyright (C) 2004-2009 John E. Davis
 
 This file is part of the S-Lang Library.
 
@@ -21,9 +21,12 @@ along with this library; if not, write to the Free Software
 Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307,
 USA.  
 */
-#define _GNU_SOURCE
+#ifndef _GNU_SOURCE
+# define _GNU_SOURCE
+#endif
 #include "slinclud.h"
 
+#include <errno.h>
 #include "slang.h"
 #include "_slang.h"
 
@@ -312,6 +315,8 @@ int SLang_handle_interrupt (void)
 {
    Interrupt_Hook_Type *h;
    int status = 0;
+   int e = errno;
+   int se = _pSLerrno_errno;
 
    h = Interrupt_Hooks;
    while (h != NULL)
@@ -321,7 +326,8 @@ int SLang_handle_interrupt (void)
 
 	h = h->next;
      }
-
+   errno = e;
+   _pSLerrno_errno = se;
    return status;
 }
 
