@@ -2,7 +2,7 @@
 /* Standard intrinsic functions for S-Lang.  Included here are string
    and array operations */
 /*
-Copyright (C) 2004, 2005, 2006, 2007, 2008 John E. Davis
+Copyright (C) 2004-2009 John E. Davis
 
 This file is part of the S-Lang Library.
 
@@ -605,6 +605,7 @@ static int intrin_get_defines (void)
 
 static void intrin_get_reference (char *name)
 {
+   if (*name == '&') name++;
    _pSLang_push_nt_as_ref (_pSLlocate_name (name));
 }
 
@@ -672,6 +673,20 @@ static int class_id_intrinsic (void)
    return _pSLclass_get_class (type)->cl_data_type;
 }
 
+static void datatype_intrinsic (SLtype *t)
+{
+   SLang_Class_Type *cl;
+
+   if (0 == SLclass_is_class_defined (*t))
+     {
+	(void) SLang_push_null ();
+	return;
+     }
+
+   cl = _pSLclass_get_class (*t);
+   (void) SLang_push_datatype (cl->cl_data_type);
+}
+
 static int do_obj_cmp_fun (int (*fun)(SLang_Object_Type *, SLang_Object_Type *))
 {
    int eqs;
@@ -1000,11 +1015,17 @@ static int is_defined_intrin (char *s)
 {
    return SLang_is_defined (s);
 }
+
 static int system_intrinsic (char *s)
 {
    return SLsystem (s);
 }
 
+static int system_intr_intrinsic (char *s)
+{
+   return SLsystem_intr (s);
+}
+
 static int stack_depth_intrin (void)
 {
    return SLstack_depth ();
@@ -1094,6 +1115,7 @@ static SLang_Intrin_Fun_Type SLang_Basic_Table [] = /*{{{*/
    MAKE_INTRINSIC_0("__is_same", is_same_intrinsic, SLANG_INT_TYPE),
    MAKE_INTRINSIC_0("__class_type", class_type_intrinsic, SLANG_INT_TYPE),
    MAKE_INTRINSIC_0("__class_id", class_id_intrinsic, SLANG_INT_TYPE),
+   MAKE_INTRINSIC_1("__datatype", datatype_intrinsic, SLANG_VOID_TYPE, SLANG_SLTYPE_INT_TYPE),
    MAKE_INTRINSIC_0("_eqs", eqs_intrinsic, SLANG_INT_TYPE),
    MAKE_INTRINSIC_S("get_doc_string_from_file",  get_doc_string_intrin, SLANG_VOID_TYPE),
    MAKE_INTRINSIC_S("add_doc_file", add_doc_file_intrin, SLANG_VOID_TYPE),
@@ -1113,6 +1135,7 @@ static SLang_Intrin_Fun_Type SLang_Basic_Table [] = /*{{{*/
    MAKE_INTRINSIC_0("dup",  do_dup, SLANG_VOID_TYPE),
    MAKE_INTRINSIC_0("integer",  intrin_integer, SLANG_VOID_TYPE),
    MAKE_INTRINSIC_S("system",  system_intrinsic, SLANG_INT_TYPE),
+   MAKE_INTRINSIC_S("system_intr",  system_intr_intrinsic, SLANG_INT_TYPE),
    MAKE_INTRINSIC_0("_apropos",  intrin_apropos, SLANG_VOID_TYPE),
    MAKE_INTRINSIC_0("_get_namespaces", intrin_get_namespaces, SLANG_VOID_TYPE),
    MAKE_INTRINSIC_S("_trace_function",  _pSLang_trace_fun, SLANG_VOID_TYPE),
@@ -1167,7 +1190,7 @@ static SLang_Intrin_Fun_Type SLang_Basic_Table [] = /*{{{*/
 #ifdef SLANG_DOC_DIR
 SLFUTURE_CONST char *SLang_Doc_Dir = SLANG_DOC_DIR;
 #else
-SLFUTURE_CONST *SLang_Doc_Dir = "";
+SLFUTURE_CONST char *SLang_Doc_Dir = "";
 #endif
 
 #ifdef SLANG_INSTALL_PREFIX
