@@ -1,5 +1,5 @@
 /*
-Copyright (C) 2004, 2005, 2006, 2007, 2008 John E. Davis
+Copyright (C) 2004-2009 John E. Davis
 
 This file is part of the S-Lang Library.
 
@@ -234,7 +234,13 @@ static SLstring_Type *find_string (SLCONST char *s, unsigned int len, unsigned l
    /* Assume it is an slstring */
    sls = find_slstring (s, hash);
    if (sls != NULL)
-     return sls;
+     {
+	/* This means that sls->bytes == s.  But the string that we are looking
+	 * for consists of just the first len bytes.  Check that too.
+	 */
+	if (sls->len == len)
+	  return sls;
+     }
 
    /* Ok, not an slstring.  Try to find a matching one */
    sls = String_Hash_Table [(unsigned int) MAP_HASH_TO_INDEX(hash)];
@@ -581,6 +587,7 @@ void _pSLunallocate_slstring (char *s, unsigned int len)
    free_sls (sls);
 }
 
+/* frees s upon error */
 char *_pSLcreate_via_alloced_slstring (char *s, unsigned int len)
 {   
    unsigned long hash;
@@ -631,18 +638,20 @@ char *_pSLcreate_via_alloced_slstring (char *s, unsigned int len)
 /* Note, a and b may be ordinary strings.  The result is an slstring */
 char *SLang_concat_slstrings (char *a, char *b)
 {
-   unsigned int lena, len;
+   unsigned int lena, lenb, len;
    char *c;
 
-   lena = strlen (a);
-   len = lena + strlen (b);
+   lena = _pSLstring_bytelen (a);
+   lenb = _pSLstring_bytelen (b);
+   len = lena + lenb;
 
    c = _pSLallocate_slstring (len);
    if (c == NULL)
      return NULL;
 
-   strcpy (c, a);
-   strcpy (c + lena, b);
+   memcpy (c, a, lena);
+   memcpy (c + lena, b, lenb);
+   c[len] = 0;
 
    return _pSLcreate_via_alloced_slstring (c, len);
 }
