@@ -1,7 +1,7 @@
 /* -*- mode: C; mode: fold; -*- */
 /* string manipulation functions for S-Lang. */
 /*
-Copyright (C) 2004, 2005, 2006, 2007, 2008 John E. Davis
+Copyright (C) 2004-2009 John E. Davis
 
 This file is part of the S-Lang Library.
 
@@ -194,9 +194,11 @@ static void strcat_cmd (void) /*{{{*/
    c1 = c;
    for (i = 0; i < nargs; i++)
      {
-	strcpy (c1, ptrs[i]);
-	c1 += _pSLstring_bytelen (ptrs[i]);   /* c1 does not take advantage of the slstring cache */
+	unsigned int len2 = _pSLstring_bytelen (ptrs[i]);
+	memcpy (c1, ptrs[i], len2);
+	c1 += len2; /* c1 does not take advantage of the slstring cache */
      }
+   *c1 = 0;
 
    free_and_return:
    for (i = 0; i < nargs; i++)
@@ -403,14 +405,14 @@ static int str_replace_cmd_1 (char *orig, char *match, char *rep, unsigned int m
 
 	next_s = strstr (s, match);    /* cannot be NULL */
 	len = (unsigned int) (next_s - s);
-	strncpy (t, s, len);
+	memcpy (t, s, len);
 	t += len;
-	strcpy (t, rep);
+	memcpy (t, rep, rep_len);
 	t += rep_len;
 	
 	s = next_s + match_len;
      }
-   strcpy (t, s);
+   strcpy (t, s);		       /* will \0 terminate t */
    *new_strp = new_str;
 
    return (int) num_replaces;
@@ -1841,6 +1843,48 @@ static int string_match_nth_cmd (int *nptr) /*{{{*/
 
 /*}}}*/
 
+static void string_matches_cmd (char *str, char *pat, int *nptr)
+{
+   int status;
+   unsigned int i;
+   unsigned int lens[10];
+   unsigned int offsets[10];
+   char **strs;
+   SLindex_Type num;
+   SLang_Array_Type *at;
+
+   status = string_match_cmd (str, pat, nptr);
+   if (status <= 0)
+     {
+	SLang_push_null ();
+	return;
+     }
+   
+   for (i = 0; i < 10; i++)
+     {
+	if (-1 == SLregexp_nth_match (Regexp, i, offsets+i, lens+i))
+	  break;
+	offsets[i] += Regexp_Match_Byte_Offset;
+     }
+
+   num = (SLindex_Type)i;
+
+   if (NULL == (at = SLang_create_array (SLANG_STRING_TYPE, 0, NULL, &num, 1)))
+     return;
+   
+   strs = (char **) at->data;
+   for (i = 0; i < (unsigned int) num; i++)
+     {
+	if (NULL == (strs[i] = SLang_create_nslstring (str+offsets[i], lens[i])))
+	  {
+	     SLang_free_array (at);
+	     return;
+	  }
+     }
+
+   (void) SLang_push_array (at, 1);
+}
+
 /* UTF-8 ok */
 static char *create_delimited_string (char **list, unsigned int n, 
 				      char *delim)
@@ -1872,12 +1916,15 @@ static char *create_delimited_string (char **list, unsigned int n,
 	
    while (num > 1)
      {
+	unsigned int len2;
+
 	while (list[i] == NULL)
 	  i++;
-	
-	strcpy (s, list[i]);
-	s += strlen (list[i]);
-	strcpy (s, delim);
+
+	len2 = strlen (list[i]);
+	memcpy (s, list[i], len2);
+	s += len2;
+	strcpy (s, delim);	       /* \0 terminates s */
 	s += dlen;
 	i++;
 	num--;
@@ -2202,6 +2249,7 @@ static SLang_Intrin_Fun_Type Strops_Table [] = /*{{{*/
    MAKE_INTRINSIC_3("extract_element", extract_element_cmd, SLANG_VOID_TYPE, SLANG_STRING_TYPE, SLANG_INT_TYPE, SLANG_WCHAR_TYPE),
    MAKE_INTRINSIC_3("is_list_element", is_list_element_cmd, SLANG_INT_TYPE, SLANG_STRING_TYPE, SLANG_STRING_TYPE, SLANG_WCHAR_TYPE),
    MAKE_INTRINSIC_SSI("string_match", string_match_cmd, SLANG_INT_TYPE),
+   MAKE_INTRINSIC_SSI("string_matches", string_matches_cmd, SLANG_VOID_TYPE),
    MAKE_INTRINSIC_I("string_match_nth", string_match_nth_cmd, SLANG_INT_TYPE),
    MAKE_INTRINSIC_0("strlow", strlow_cmd, SLANG_VOID_TYPE),
    MAKE_INTRINSIC_1("tolower", tolower_cmd, SLANG_INT_TYPE, SLANG_WCHAR_TYPE),
