@@ -1,6 +1,6 @@
 /* Structure type implementation */
 /*
-Copyright (C) 2004, 2005, 2006, 2007, 2008 John E. Davis
+Copyright (C) 2004-2009 John E. Davis
 
 This file is part of the S-Lang Library.
 
@@ -27,7 +27,7 @@ USA.
 #include "_slang.h"
 
 
-void SLang_free_struct (_pSLang_Struct_Type *s)
+static void free_struct (_pSLang_Struct_Type *s)
 {
    _pSLstruct_Field_Type *field, *field_max;
 
@@ -72,6 +72,11 @@ void SLang_free_struct (_pSLang_Struct_Type *s)
    SLfree ((char *) s);
 }
 
+void SLang_free_struct (_pSLang_Struct_Type *s)
+{
+   free_struct (s);
+}
+
 static _pSLang_Struct_Type *allocate_struct (unsigned int nfields)
 {
    _pSLang_Struct_Type *s;
@@ -138,7 +143,7 @@ int SLang_pop_struct (_pSLang_Struct_Type **sp)
    if (type != SLANG_STRUCT_TYPE)
      {
 	cl = _pSLclass_get_class (type);
-	if (cl->cl_struct_def == NULL)
+	if (cl->is_struct == 0)
 	  {
 	     *sp = NULL;
 	     SLang_free_object (&obj);
@@ -678,6 +683,8 @@ typedef struct _Struct_Info_Type
    /* Other methods */
    SLang_Name_Type *destroy_method;
    SLang_Name_Type *string_method;
+   SLang_Name_Type *aget_method;
+   SLang_Name_Type *aput_method;
 }
 Struct_Info_Type;
 
@@ -1239,6 +1246,95 @@ static void add_string_method (SLtype *typep, SLang_Ref_Type *ref)
    si->string_method = SLang_copy_function (f);
 }
 
+static int aget_method (SLtype type, unsigned int num_indices)
+{
+   Struct_Info_Type *si;
+   
+   if (NULL == (si = find_struct_info (type, 1)))
+     return -1;
+   
+   if (si->aget_method == NULL)
+     {
+	SLang_verror (SL_Internal_Error, "aget method called but is NULL");
+	return -1;
+     }
+
+   if ((-1 == _pSLang_restart_arg_list ((int) num_indices))
+       || (-1 == SLang_end_arg_list ())
+       || (-1 == SLexecute_function (si->aget_method)))
+     return -1;
+   
+   return 0;
+}
+
+static int aput_method (SLtype type, unsigned int num_indices)
+{
+   Struct_Info_Type *si;
+   
+   if (NULL == (si = find_struct_info (type, 1)))
+     return -1;
+   
+   if (si->aput_method == NULL)
+     {
+	SLang_verror (SL_Internal_Error, "aput method called but is NULL");
+	return -1;
+     }
+
+   if ((-1 == _pSLang_restart_arg_list ((int) num_indices))
+       || (-1 == SLang_end_arg_list ())
+       || (-1 == SLexecute_function (si->aput_method)))
+     return -1;
+
+   return 0;
+}
+
+
+static void add_aget_method (SLtype *typep, SLang_Ref_Type *ref)
+{
+   Struct_Info_Type *si;
+   SLang_Name_Type *f;
+   SLtype type = *typep;
+   SLang_Class_Type *cl;
+
+   if (NULL == (cl = _pSLclass_get_class (type)))
+     return;
+
+   if (NULL == (f = SLang_get_fun_from_ref (ref)))
+     return;
+
+   if (NULL == (si = find_struct_info (type, 1)))
+     return;
+   
+   if (si->aget_method != NULL)
+     SLang_free_function (si->aget_method);
+
+   si->aget_method = SLang_copy_function (f);
+   (void) SLclass_set_aget_function (cl, aget_method);
+}
+
+static void add_aput_method (SLtype *typep, SLang_Ref_Type *ref)
+{
+   Struct_Info_Type *si;
+   SLang_Name_Type *f;
+   SLtype type = *typep;
+   SLang_Class_Type *cl;
+
+   if (NULL == (cl = _pSLclass_get_class (type)))
+     return;
+
+   if (NULL == (f = SLang_get_fun_from_ref (ref)))
+     return;
+
+   if (NULL == (si = find_struct_info (type, 1)))
+     return;
+   
+   if (si->aput_method != NULL)
+     SLang_free_function (si->aput_method);
+
+   si->aput_method = SLang_copy_function (f);
+   (void) SLclass_set_aput_function (cl, aput_method);
+}
+
 static Typecast_Info_Type *find_typecast (Struct_Info_Type *si, SLtype to)
 {
    Typecast_Info_Type *ti = si->ti;
@@ -1358,6 +1454,32 @@ static int struct_sput (SLtype type, SLFUTURE_CONST char *name)
    return 0;
 }
 
+int _pSLstruct_pop_field (SLang_Struct_Type *s, SLFUTURE_CONST char *name, int do_free)
+{
+   int ret = pop_to_struct_field (s, name);
+
+   if (do_free)
+     SLang_free_struct (s);
+
+   return ret;
+}
+
+int _pSLstruct_push_field (SLang_Struct_Type *s, SLFUTURE_CONST char *name, int do_free)
+{
+   _pSLstruct_Field_Type *f;
+   int ret;
+
+   if (NULL == (f = pop_field (s, name, find_field)))
+     {
+	if (do_free) SLang_free_struct (s);
+	return -1;
+     }
+
+   ret = _pSLpush_slang_obj (&f->obj);
+   if (do_free) SLang_free_struct (s);
+   return ret;
+}
+
 static int struct_sget (SLtype type, SLFUTURE_CONST char *name)
 {
    _pSLang_Struct_Type *s;
@@ -1475,6 +1597,17 @@ static int struct_eqs_method (SLtype a_type, VOID_STAR ap, SLtype b_type, VOID_S
    return 1;
 }
 
+static int struct_acopy (SLtype unused, VOID_STAR src_sptr, VOID_STAR dest_sptr)
+{
+   _pSLang_Struct_Type *s;
+   
+   (void) unused;
+   s = *(_pSLang_Struct_Type **)src_sptr;
+   s->num_refs++;
+   *(_pSLang_Struct_Type **)dest_sptr = s;
+   return 0;
+}
+
 int _pSLstruct_define_typedef (void)
 {
    char *type_name;
@@ -1519,10 +1652,12 @@ int _pSLstruct_define_typedef (void)
 
    (void) SLclass_set_string_function (cl, string_method);
    (void) SLclass_set_eqs_function (cl, struct_eqs_method);
+   (void) SLclass_set_acopy_function (cl, struct_acopy);
 
    cl->cl_sget = struct_sget;
    cl->cl_sput = struct_sput;
    cl->is_container = 1;
+   cl->is_struct = 1;
 
    if ((-1 == SLclass_register_class (cl,
 				     SLANG_VOID_TYPE,   /* any open slot */
@@ -1590,8 +1725,10 @@ static int register_struct (void)
    cl->cl_sput = struct_sput;
    (void) SLclass_set_string_function (cl, string_method);
    (void) SLclass_set_eqs_function (cl, struct_eqs_method);
+   (void) SLclass_set_acopy_function (cl, struct_acopy);
 
    cl->is_container = 1;
+   cl->is_struct = 1;
 
    if (-1 == SLclass_register_class (cl, SLANG_STRUCT_TYPE, sizeof (_pSLang_Struct_Type),
 				     SLANG_CLASS_TYPE_PTR))
@@ -1789,6 +1926,8 @@ static SLang_Intrin_Fun_Type Struct_Table [] =
    MAKE_INTRINSIC_0("__add_binary", add_binary_op_intrin, SLANG_VOID_TYPE),
    MAKE_INTRINSIC_0("__add_destroy", add_destroy_method, SLANG_VOID_TYPE),
    MAKE_INTRINSIC_2("__add_string", add_string_method, SLANG_VOID_TYPE, SLANG_DATATYPE_TYPE, SLANG_REF_TYPE),
+   MAKE_INTRINSIC_2("__add_aget", add_aget_method, SLANG_VOID_TYPE, SLANG_DATATYPE_TYPE, SLANG_REF_TYPE),
+   MAKE_INTRINSIC_2("__add_aput", add_aput_method, SLANG_VOID_TYPE, SLANG_DATATYPE_TYPE, SLANG_REF_TYPE),
    MAKE_INTRINSIC_3("__add_typecast", add_typecast_method, SLANG_VOID_TYPE, SLANG_DATATYPE_TYPE, SLANG_DATATYPE_TYPE, SLANG_REF_TYPE),
 
    /* MAKE_INTRINSIC_I("_create_struct", create_struct, SLANG_VOID_TYPE), */
