@@ -3,7 +3,7 @@
  */
 
 /*
-Copyright (C) 2004, 2005, 2006, 2007, 2008 John E. Davis
+Copyright (C) 2004-2009 John E. Davis
 
 This file is part of the S-Lang Library.
 
@@ -1101,7 +1101,7 @@ static int tcap_getent (SLCONST char *term, SLterminfo_Type *ti)
 	       {
 		  SLwchar_Type wch;
 
-		  t = (unsigned char *) _pSLexpand_escaped_char ((char *) t, &wch, NULL);
+		  t = (unsigned char *) _pSLexpand_escaped_char ((char *) t, (char *) tmax, &wch, NULL);
 		  if (t == NULL)
 		    {
 		       SLfree ((char *)buf);
