@@ -1,6 +1,6 @@
 /* Basic type operations for S-Lang */
 /*
-Copyright (C) 2004, 2005, 2006, 2007, 2008 John E. Davis
+Copyright (C) 2004-2009 John E. Davis
 
 This file is part of the S-Lang Library.
 
@@ -67,6 +67,7 @@ int _pSLang_push_slstring (char *s)
    return -1;
 }
 
+/* Frees s upon error */
 int _pSLpush_alloced_slstring (char *s, unsigned int len)
 {
    if (NULL == (s = _pSLcreate_via_alloced_slstring (s, len)))
@@ -356,125 +357,6 @@ static int string_acopy (SLtype unused, VOID_STAR src_sptr, VOID_STAR dest_sptr)
    return 0;
 }
 
-struct _pSLang_Foreach_Context_Type
-{
-   char *string;
-   unsigned char *s, *smax;	       /* pointers into string */
-   int using_chars;
-};
-
-static SLang_Foreach_Context_Type *
-string_foreach_open (SLtype type, unsigned int num)
-{
-   char *s;
-   char *u;
-   SLang_Foreach_Context_Type *c;
-   int using_chars = 0;
-
-   (void) type;
-
-   if (-1 == SLang_pop_slstring (&s))
-     return NULL;
-
-   switch (num)
-     {
-      case 1:
-	if (-1 == SLang_pop_slstring (&u))
-	  {
-	     SLang_free_slstring (s);
-	     return NULL;
-	  }
-	if (0 == strcmp (u, "chars"))
-	  using_chars = 1;
-	else if (0 == strcmp (u, "bytes"))
-	  using_chars = 0;
-	else
-	  {
-	     _pSLang_verror (SL_InvalidParm_Error, "Expected foreach (String_Type) using (chars|bytes)");
-	     SLang_free_slstring (u);
-	     SLang_free_slstring (s);
-	     return NULL;
-	  }
-	SLang_free_slstring (u);
-   	break;
-
-      case 0:
-	using_chars = 0;
-	break;
-      default:
-	_pSLang_verror (SL_NumArgs_Error,
-		      "'foreach (String_Type) using' requires single control value (chars|bytes)");
-	return NULL;
-     }
-   
-   /* In UTF-8 mode, chars and bytes are synonymous */
-   if (_pSLinterp_UTF8_Mode == 0)
-     using_chars = 0;
-
-   c = (SLang_Foreach_Context_Type *)SLmalloc (sizeof (SLang_Foreach_Context_Type));
-   if (c == NULL)
-     {
-	SLang_free_slstring (s);
-	return NULL;
-     }
-
-   memset ((char *) c, 0, sizeof (SLang_Foreach_Context_Type));
-   c->string = s;
-   c->s = (unsigned char *) s;
-   c->smax = (unsigned char *)s + strlen (s);
-   c->using_chars = using_chars;
-   return c;
-}
-
-static void string_foreach_close (SLtype type, SLang_Foreach_Context_Type *c)
-{
-   (void) type;
-   if (c == NULL) return;
-   SLang_free_slstring (c->string);
-   SLfree ((char *) c);
-}
-
-static int string_foreach (SLtype type, SLang_Foreach_Context_Type *c)
-{
-   unsigned char ch;
-   SLwchar_Type wch;
-   unsigned char *s, *s1, *smax;
-   
-   (void) type;
-
-   s = c->s;
-   smax = c->smax;
-   if (s == smax)
-     return 0;
-
-   if (c->using_chars == 0)
-     {
-	ch = (unsigned char) *s++;
-	c->s = s;
-
-	if (-1 == SLclass_push_char_obj (SLANG_UCHAR_TYPE, ch))
-	  return -1;
-
-	return 1;
-     }
-   s1 = SLutf8_decode (s, smax, &wch, NULL);
-   if (s1 == NULL)
-     {
-	int iwch = (int) *s;
-	c->s = s + 1;
-	/* Invalid encoded char-- return it as a negative int */
-	if (-1 == SLang_push_int (-iwch))
-	  return -1;
-	
-	return 1;
-     }
-   c->s = s1;
-   if (-1 == SLang_push_wchar (wch))
-     return -1;
-   
-   return 1;
-}
-
 SLang_Array_Type *_pSLstrings_to_array (char **strs, unsigned int n)
 {
    char **data;
@@ -1043,14 +925,26 @@ static int anytype_apush (SLtype type, VOID_STAR ptr)
 /* SLANG_INTP_TYPE */
 static int intp_push (SLtype unused, VOID_STAR ptr)
 {
+   int *addr;
    (void) unused;
-   return SLclass_push_int_obj (SLANG_INT_TYPE, **(int **)ptr);
+   addr = *(int **)ptr;
+   if (addr == NULL)
+     return SLclass_push_int_obj (SLANG_INT_TYPE, 0);
+   return SLclass_push_int_obj (SLANG_INT_TYPE, *addr);
 }
 
 static int intp_pop (SLtype unused, VOID_STAR ptr)
 {
+   int *addr;
+
    (void) unused;
-   return SLang_pop_integer (*(int **) ptr);
+   addr = *(int **)ptr;
+   if (addr == NULL)
+     {
+	SLang_verror (SL_VariableUninitialized_Error, "_IntegerP_Type: integer pointer address is NULL");
+	return -1;
+     }
+   return SLang_pop_integer (addr);
 }
 
 static int undefined_method (SLtype t, VOID_STAR p)
@@ -1086,8 +980,8 @@ int _pSLregister_types (void)
    (void) SLclass_set_push_function (cl, undefined_method);
    (void) SLclass_set_pop_function (cl, undefined_method);
    (void) SLclass_set_destroy_function (cl, void_undefined_method);
-   if (-1 == SLclass_register_class (cl, SLANG_UNDEFINED_TYPE, sizeof (char *),
-				     SLANG_CLASS_TYPE_PTR))
+   if (-1 == SLclass_register_class (cl, SLANG_UNDEFINED_TYPE, sizeof (int),
+				     SLANG_CLASS_TYPE_SCALAR))
      return -1;
    /* Make Void_Type a synonym for Undefined_Type.  Note that this does 
     * not mean that Void_Type represents SLANG_VOID_TYPE.  Void_Type is 
@@ -1099,12 +993,12 @@ int _pSLregister_types (void)
    if (-1 == _pSLarith_register_types ())
      return -1;
 
-   /* SLANG_INTP_TYPE */
+   /* SLANG_INTP_TYPE -- not used within the interpreter */
    if (NULL == (cl = SLclass_allocate_class ("_IntegerP_Type")))
      return -1;
    (void) SLclass_set_push_function (cl, intp_push);
    (void) SLclass_set_pop_function (cl, intp_pop);
-   if (-1 == SLclass_register_class (cl, SLANG_INTP_TYPE, sizeof (int),
+   if (-1 == SLclass_register_class (cl, SLANG_INTP_TYPE, sizeof (int *),
 				     SLANG_CLASS_TYPE_SCALAR))
      return -1;
 
@@ -1115,9 +1009,9 @@ int _pSLregister_types (void)
    (void) SLclass_set_destroy_function (cl, string_destroy);
    (void) SLclass_set_push_function (cl, string_push);
    (void) SLclass_set_acopy_function (cl, string_acopy);
-   cl->cl_foreach_open = string_foreach_open;
-   cl->cl_foreach_close = string_foreach_close;
-   cl->cl_foreach = string_foreach;
+   cl->cl_foreach_open = _pSLbstring_foreach_open;
+   cl->cl_foreach_close = _pSLbstring_foreach_close;
+   cl->cl_foreach = _pSLbstring_foreach;
    cl->cl_cmp = string_cmp;
    if (-1 == SLclass_register_class (cl, SLANG_STRING_TYPE, sizeof (char *),
 				     SLANG_CLASS_TYPE_PTR))
