@@ -1,26 +1,34 @@
 #include <stdio.h>
 #include <stdio.h>
-#include "sl-feat.h"
+#include "slinclud.h"
 #include "slang.h"
 #include "_slang.h"
 
-static void dump_token (_SLang_Token_Type *t)
+static void dump_token (_pSLang_Token_Type *t)
 {
    char buf [256], *b;
    
    b = buf;
    
-   if (SLang_Error)
+   if (_pSLang_Error)
      return;
    
    switch (t->type)
      {
+      case TIMES_TOKEN:
+	b = "*";
+	break;
       case IDENT_TOKEN:
 	b = t->v.s_val;
 	break;
 	
+      case TMP_TOKEN:
+	b = "__tmp"; break;
+
       case CHAR_TOKEN:
+      case UCHAR_TOKEN:
       case INT_TOKEN:
+      case LONG_TOKEN:
 	sprintf (buf, "%ld", t->v.long_val);
 	break;
 	
@@ -32,8 +40,12 @@ static void dump_token (_SLang_Token_Type *t)
 	sprintf (buf, "\"%s\"", t->v.s_val);
 	break;
 	
+      case STRING_DOLLAR_TOKEN:
+	sprintf (buf, "\"%s\"$", t->v.s_val);
+	break;
+	
       case PLUSPLUS_TOKEN:
-	sprintf (buf, "++%s", t->v.s_val);
+	sprintf (buf, "++ (RPN)");
 	break;
 	
       case POST_PLUSPLUS_TOKEN:
@@ -41,7 +53,7 @@ static void dump_token (_SLang_Token_Type *t)
 	break;
 	
       case MINUSMINUS_TOKEN:
-	sprintf (buf, "--%s", t->v.s_val);
+	sprintf (buf, "-- (RPN)");
 	break;
 	
       case POST_MINUSMINUS_TOKEN:
@@ -57,7 +69,7 @@ static void dump_token (_SLang_Token_Type *t)
 	break;
 	
       case ASSIGN_TOKEN:
-	sprintf (buf, "=%s", t->v.s_val);
+	sprintf (buf, "= (RPN assign)");
 	break;
 	
       case EOF_TOKEN:
@@ -95,11 +107,11 @@ static void dump_token (_SLang_Token_Type *t)
       case IFNOT_TOKEN:
 	b = "!if";
 	break;
-	
+#ifdef ABS_TOKEN
       case ABS_TOKEN:
 	b = "abs";
 	break;
-	
+#endif
       case LT_TOKEN:
 	b = "<";
 	break;
@@ -167,19 +179,19 @@ static void dump_token (_SLang_Token_Type *t)
       case SHL_TOKEN:
 	b = "shl";
 	break;
-	
+#ifdef SQR_TOKEN	
       case SQR_TOKEN:
 	b = "sqr";
 	break;
-	
+#endif
       case CASE_TOKEN:
 	b = "case";
 	break;
-	
+#ifdef SIGN_TOKEN	
       case SIGN_TOKEN:
 	b = "sign";
 	break;
-	
+#endif
       case BREAK_TOKEN:
 	b = "break";
 	break;
@@ -187,11 +199,27 @@ static void dump_token (_SLang_Token_Type *t)
       case STATIC_TOKEN:
 	b = "static";
 	break;
+
+      case PRIVATE_TOKEN:
+	b = "private";
+	break;
 	
+      case PUBLIC_TOKEN:
+	b = "public";
+	break;
+
       case STRUCT_TOKEN:
 	b = "struct";
 	break;
 	
+      case STRUCT_WITH_ASSIGN_TOKEN:
+	b = "__struct_with_assign";
+	break;
+	
+      case QUALIFIER_TOKEN:
+	b = "__qualifiers";
+	break;
+	
       case RETURN_TOKEN:
 	b = "return";
 	break;
@@ -239,19 +267,23 @@ static void dump_token (_SLang_Token_Type *t)
       case ELSE_TOKEN:
 	b = "else";
 	break;
-	
+#ifdef MUL2_TOKEN	
       case MUL2_TOKEN:
 	b = "mul2";
 	break;
-	
+#endif
       case DEFINE_TOKEN:
-	b = ")";
+	sprintf (buf, ") %s", t->v.s_val);
 	break;
 
       case DEFINE_STATIC_TOKEN:
-	b = ")static";
+	sprintf (buf, ") static %s", t->v.s_val);
 	break;
 	
+      case DEFINE_PRIVATE_TOKEN:
+	sprintf (buf, ") private %s", t->v.s_val);
+	break;
+
       case LOOP_TOKEN:
 	b = "loop";
 	break;
@@ -308,14 +340,26 @@ static void dump_token (_SLang_Token_Type *t)
 	sprintf (buf, "%s--", t->v.s_val);
 	break;
 
+      case _ARRAY_DIVEQS_TOKEN:
+	sprintf (buf, "/= (array)"); break;
+
+#if 0
       case _DEREF_ASSIGN_TOKEN:
 	sprintf (buf, "=@%s", t->v.s_val);
 	break;
-
+#endif
       case _REF_TOKEN:
 	sprintf (buf, "%s __ref", t->v.s_val);
 	break;
 	
+      case _ARRAY_ELEM_REF_TOKEN:
+	b = "__array_elem_ref";
+	break;
+	
+      case _STRUCT_FIELD_REF_TOKEN:
+	sprintf (buf, "%s __struct_field_ref", t->v.s_val);
+	break;
+
       case ORELSE_TOKEN:
 	b = "orelse";
 	break;
@@ -372,6 +416,13 @@ static void dump_token (_SLang_Token_Type *t)
 	b = ":";
 	break;
 	
+      case SC_AND_TOKEN: b = "&&"; break;
+      case SC_OR_TOKEN: b = "||"; break;
+
+      case POW_TOKEN:
+	b = "^";
+	break;
+
       case ADD_TOKEN:
 	b = "+";
 	break;
@@ -392,15 +443,11 @@ static void dump_token (_SLang_Token_Type *t)
 	b = "__aget";
 	break;
 	
-      case _ARRAY_ASSIGN_TOKEN:
-	b = "__aput";
-	break;
-	
       case DOT_TOKEN:
 	sprintf (buf, "%s .", t->v.s_val);
 	break;
 
-      case METHOD_TOKEN:
+      case DOT_METHOD_CALL_TOKEN:
 	sprintf (buf, "%s __eargs __method_call", t->v.s_val);
 	break;
 
@@ -419,12 +466,66 @@ static void dump_token (_SLang_Token_Type *t)
       case _STRUCT_POST_MINUSMINUS_TOKEN:
 	b = "__struct_pminusminus"; break;
 
+      case _ARRAY_ASSIGN_TOKEN:
+	b = "_aput"; break;
+      case _ARRAY_PLUSEQS_TOKEN:
+	b = "__array_pluseqs"; break;
+      case _ARRAY_MINUSEQS_TOKEN:
+	b = "__array_minuseqs"; break;
+      case _ARRAY_PLUSPLUS_TOKEN:
+	b = "__array_plusplus"; break;
+      case _ARRAY_POST_PLUSPLUS_TOKEN:
+	b = "__array_pplusplus"; break;
+      case _ARRAY_MINUSMINUS_TOKEN:
+	b = "__array_minusminus"; break;
+      case _ARRAY_POST_MINUSMINUS_TOKEN:
+	b = "__array_pminusminus"; break;
+
+      case _DEREF_ASSIGN_TOKEN:
+	b = "__deref_eqs"; break;
+      case _DEREF_PLUSEQS_TOKEN:
+	b = "__deref_pluseqs"; break;
+      case _DEREF_MINUSEQS_TOKEN:
+	b = "__deref_minuseqs"; break;
+      case _DEREF_PLUSPLUS_TOKEN:
+	b = "__deref_plusplus"; break;
+      case _DEREF_POST_PLUSPLUS_TOKEN:
+	b = "__deref_pplusplus"; break;
+      case _DEREF_MINUSMINUS_TOKEN:
+	b = "__deref_minusminus"; break;
+      case _DEREF_POST_MINUSMINUS_TOKEN:
+	b = "__deref_pminusminus"; break;
+	
+      case _DEREF_FUNCALL_TOKEN:
+	b = "__deref_funcall"; break;
+
       case _NULL_TOKEN: b = "NULL"; break;
 	
       case USING_TOKEN:
 	b = "__using__"; break;
       case FOREACH_TOKEN:
 	b = "__foreach__"; break;
+	
+      case LOOP_THEN_TOKEN:
+	b = "__loopthen__"; break;
+#if 0
+      case LOOP_ELSE_TOKEN:
+	b = "__loopelse__"; break;
+#endif	
+      case TRY_TOKEN:
+	b = "try"; break;
+	
+      case LINE_NUM_TOKEN:
+        sprintf (buf, "__line_num__ %ld", t->v.long_val); break;
+
+      case _INLINE_WILDCARD_ARRAY_TOKEN:
+	sprintf (buf, "__inline_wildcard_array__"); break;
+
+      case BOS_TOKEN:
+	b = "__bos__"; break;
+      case EOS_TOKEN:
+	b = "__eos__"; break;
+
 
       default:
 	sprintf (buf, "____UNKNOWN___0x%X", t->type);
@@ -450,8 +551,10 @@ int main (int argc, char **argv)
    if (-1 == SLang_init_slang ())
      return 1;
    
-   _SLcompile_ptr = dump_token;
+   _pSLcompile_ptr = dump_token;
+   _pSLang_Compile_BOSEOS = 3;
+   _pSLang_Compile_BOFEOF = 1;
    SLang_load_file (file);
    
-   return SLang_Error;
+   return _pSLang_Error;
 }
